/***************************************************************************
                          ccmdprocessor.h  -  command processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jul 5 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CCMDPROCESSOR_H
#define CCMDPROCESSOR_H

#include <qobject.h>

#include <list>
#include <kextsock.h>

#include "csessionbase.h"

class cEventNotification;

using namespace std;

/**
class cCmdProcessor handles processing of command after ENTER has been
pressed - it splits it into individual commands, checks every command
against all aliases, applying them if necessary (including calling of macros),
then it gives it to cSendCmd for sending...

Whole strings can now be used as cmdseparator, macro caller and friends...
It is caller's responsibility to verify correctness of supplied strings,
that is, none of them is a prefix of another one, single-char and multi-char
strings do not collide and so... Behaviour is not defined for strings
that violate these rules.
  *@author Tomas Mecir
  */

class cCmdProcessor : public QObject, public cSessionBase {
   Q_OBJECT
public: 
  cCmdProcessor();
  ~cCmdProcessor();

  /** backslash (\) expansion yes/no */
  void setExpandBackslashes (bool val) { expandbackslashes = val; };
  /** backslash expansion in aliases/triggers only; only applies if
  backslash expansion is on */
  void setBackslashesOnlyInAT (bool val) { backslashAT = val; };
  
  void setCmdSeparatorString (QString str);
  void setSpeedWalkString (QString str);
  void setScriptString (QString str);
  void setMultiCommandString (QString str);
  void setFocusCommandString (QString str);
  void setNoParseString (QString str);
  
  void setAllowEmptyWalkStr (bool val);

  /** set spaces stripping */
  void setStripSpaces (bool strip) { stripSpaces = strip; };
signals:
  void newCommand (const QString &);
  
public slots:
  /** process input received from anywhere but the trigger */
  void processInput (const QString &text);
  /** Process input received from some trigger. We need two separate
    functions because it's possible to limit backslash expansion to
    aliases/triggers only. Aliases are expanded inside that function, so
    we don't need another separate function for these. */
  void processTriggerInput (const QString &text);
  /** enable/disable command parsing */
  void setParsing (bool value);

  /** information sent to external IP port */
  void portNotified (cEventNotification *event_notify);

protected:
  /** send information to external IP port */
  void doNotify(const int  port,
                QString   &ip_data);

  /** get next command from the input line or alias/trigger text*/
  bool getNext (QString &from, QString &command, bool isAliasTrigger = false);

  /** called by processInput and processTrigger */
  void doProcessInput (const QString &next, bool isAliasTrigger,
      bool expandAliases = true);

  /** Looks to see if this is a tab/window command call. If it is, the command
  is split into :window:mytext */
  int isFocusCommand (const QString &command);
  /** Send <command> to <window> */
  int processFocusCommand (const QString &command, int pos);
  
  /** if this is a speed-walk command, returns length of speed-walk char
  or string (it returns 0 if it isn't speedwalk) */
  int isSpeedWalkCommand (QString command);
  /** if the command is speed-walk, this will process it, starting
  at position <pos> (computed by isSpeedWalkCommand */
  void processSpeedWalk (const QString &command, int pos);
  
  /** Looks if this is a macro call. If it is, then it also splits the
  command into parts (macro name and parameters).
  Note that macros are NOT scripts, although scripts are called using
  macros.*/
  bool isMacroCall (const QString &command, QString &mname,
    QString &params, bool isAliasTrigger);
  /** Evaluate macro <mname> with parameters <params> */
  void processMacroCall (const QString &mname, const QString &params,
      bool isAliasTrigger);
  /** Runs script <sname> with parameters <params> */
  void runScript (const QString &sname, const QString &params,
      bool isAliasTrigger);

  /** handles repeater (multi-command) character. !!! modifies
  parameter command if there is a repeater !!!
  @param command Where to look for the repeater, repeater sequence is
  stripped off the command, if any */
  unsigned int repeater (QString &command);

  /** will we expand backslashes? */
  bool willExpandBS (bool isAliasTrigger);
  
  /** fix (=remove) backslashes in command */
  void fixBackslashes (QString &command, bool isAliasTrigger);
  /** protect BSs before alias expansion */
  void protectBackslashes (QString &command, bool isAliasTrigger);
  /** unprotect those BSs */
  void unprotectBackslashes (QString &command, bool isAliasTrigger);
  
  bool parsing, expandbackslashes, backslashAT;
  bool stripSpaces;
  QString separstr, walkstr, scriptstr, multistr, focusstr, noparsestr;
  bool allowemptywalkstr;
  
  list <cEventNotification *> notifications;
};



/**
class cEventNotification is used to send notification data to externally running scripts using 
the non-blocking TCP/IP functionality of QSocket

  *@author Alex Bache
  */

class cEventNotification: public QObject
{
Q_OBJECT

   private:
     KExtendedSocket *sock;
     int              port;
     list<QString>    data_list;
     
   public slots:
     // Connected to IP port successfully
     void connected();
     
     // Wrote some data to the IP port
     void wroteAll();
     
     // Connection closed
     void connectionClosed();
     
     // An error occured
     void error();
     
   signals:
     // Communication with IP port finished
     void finished(cEventNotification *item);
     
   public:
     cEventNotification(const int      ip_port,
                        const QString &ip_data);
     
     ~cEventNotification();
     
     // Send the information to the specified IP port
     void send();
     
     // Which port are we sending information to?
     int port_num() const   { return port; }
     
     // Queue data for sending at next opportunity
     void queue(const QString &ip_data);
     
}; // cEventNotification
  
  
#endif
