/***************************************************************************
                    cconnprefs.cpp  -  connection preferences, aliases, ...
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po Jul 22 2002
    copyright            : (C) 2002-2004 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>

#include "cconnprefs.h"

#include "caliaslist.h"
#include "ctriggerlist.h"
#include "cactionlist.h"
#include "cgaugelist.h"
#include "cgrouplist.h"
#include "cscriptlist.h"
#include "crunninglist.h"
#include "ctimerlist.h"
#include "cshortcutlist.h"
#include "cstatusvarlist.h"
#include "cvariablelist.h"
#include "cvartriggerlist.h"
#include "cwindowlist.h"

#include "cansiparser.h"
#include "cconsole.h"
#include "cpluginmanager.h"
#include "csession.h"
#include "cstatus.h"
#include "ctelnet.h"
#include "kmuddy.h"

#include <qdir.h>
#include <kstandarddirs.h>
#include <kconfig.h>

extern KMuddy *kmuddy;

cConnPrefs::cConnPrefs (QString name, cSession *_sess, bool loadFull)
{
  sess = _sess;

  loadedFull = loadFull;
  //do NOT load everything if there is no cSession
  if ((loadedFull) && (sess == NULL))
    loadedFull = false;

  _name = name;

  //first construct dirname
  directory = locateLocal ("appdata", "profiles/" + _name + "/");
}

cConnPrefs::~cConnPrefs()
{
  //save all preferences
  save ();

  //delete associated objects
  if (loadedFull)
  {
    delete gaugelist;
    gaugelist = 0;
    delete statusvarlist;
    statusvarlist = 0;
    delete vartriglist;
    vartriglist = 0;
    delete variablelist;
    variablelist = 0;
    delete shortcutlist;
    shortcutlist = 0;
    timerlist->disableTimers ();
    delete timerlist;
    timerlist = 0;
    delete runninglist;
    runninglist = 0;
    delete scriptlist;
    scriptlist = 0;
    delete triggergrouplist;
    triggergrouplist = 0;
    delete aliasgrouplist;
    aliasgrouplist = 0;
    delete actionlist;
    actionlist = 0;
    delete triggerlist;
    triggerlist = 0;
    delete aliaslist;
    aliaslist = 0;
    delete windowlist;
    windowlist = 0;
  }

  //reset settings
  if (sess != NULL)
    sess->ansiparser()->setUseAnsi (true);
  //IMPORTANT: we CANNOT reset telnet's quitcommand here, because
  //disconnect() (where it's needed) is called AFTER this one...
}

void cConnPrefs::load ()
{
  //prepare configfile
  KConfig *config = new KConfig (directory + "preferences", true);

  //load basic info
  config->setGroup ("General");
  setServer (config->readEntry ("Server", ""));
  setPort (config->readNumEntry ("Port", 0));
  setLogin (config->readEntry ("Login", ""));
  setPassword (config->readEntry ("Password", ""));

  //login sequence
  config->setGroup ("Login sequence");
  _connstr.clear ();
  int lsc = config->readNumEntry ("Count", 0);
  for (int i = 1; i <= lsc; i++)
  {
    QString s = config->readEntry ("Line " + QString::number(i), QString::null);
    _connstr.append (s);    
  }
  if (lsc == 0) //default login sequence
  {
    _connstr.append ("$name");
    _connstr.append ("$password");
  }
  
  //connection parameters
  config->setGroup ("Connection");
  setAnsiColors (config->readBoolEntry ("ANSI Colors", true));
  setLimitTriggers (config->readBoolEntry ("Limit triggers", true));
  setLimitRepeater (config->readBoolEntry ("Limit repeater", true));
  setNegotiateOnStartup (config->readBoolEntry ("Negotiate on startup", true));
  setPromptLabel (config->readBoolEntry ("Prompt label", false));
  setLPMudStyle (config->readBoolEntry ("LPMud style", false));
  setStatusPrompt (config->readBoolEntry ("Status prompt", false));
  setConsolePrompt (config->readBoolEntry ("Console prompt", true));
  setAutoAdvTranscript (config->readBoolEntry ("Auto logging", false));

  //commands
  config->setGroup ("Commands");
  setCmd (North, config->readEntry ("North", "n"));
  setCmd (NorthEast, config->readEntry ("NorthEast", "ne"));
  setCmd (East, config->readEntry ("East", "e"));
  setCmd (SouthEast, config->readEntry ("SouthEast", "se"));
  setCmd (South, config->readEntry ("South", "s"));
  setCmd (SouthWest, config->readEntry ("SouthWest", "sw"));
  setCmd (West, config->readEntry ("West", "w"));
  setCmd (NorthWest, config->readEntry ("NorthWest", "nw"));
  setCmd (Up, config->readEntry ("Up", "u"));
  setCmd (Down, config->readEntry ("Down", "d"));
  setQuit (config->readEntry ("Quit", "quit"));

  //scripts
  config->setGroup ("Scripts");
  setScriptDir (config->readEntry ("Script directory", QDir::homeDirPath()));
  setWorkDir (config->readEntry ("Working directory", QDir::homeDirPath()));

  //directories (the above two should go here, but we don't want to corrupt
  // existing settings)
  config->setGroup ("Directories");
  setTranscriptDir (config->readEntry ("Transcript directory", QDir::homeDirPath()));
  
#ifdef HAVE_SOUND
  //MSP
  config->setGroup ("Sound Protocol");
  sounddirs.clear ();
  setUseMSP (config->readBoolEntry ("Use MSP", true));
  setAlwaysMSP (config->readBoolEntry ("Always MSP", false));
  setMidlineMSP (config->readBoolEntry ("Midline MSP", false));
  int count = config->readNumEntry ("Path count", -1);
  if (count == -1)
  {
    //default sound dirs
    count = 1;
    sounddirs << QDir::homeDirPath() + "/sounds";
  }
  else
    for (int i = 1; i <= count; i++)
      sounddirs << config->readEntry (QString("Path ") + QString::number(i),
            QString::null);
#endif  //HAVE_SOUND

#ifdef HAVE_MXP
  //MXP
  config->setGroup ("MUD Extension Protocol");
  //how to use MXP? By default, we auto-detect it (i.e., we parse for MXP stuff, but we
  //remain in LOCKED mode by default, until changed by server)
  setUseMXP (config->readNumEntry ("Use MXP", 3));
  varprefix = config->readEntry ("Variable prefix", "mxp");
#endif

  delete config;

  aliaslist = 0;
  triggerlist = 0;
  actionlist = 0;
  aliasgrouplist = 0;
  triggergrouplist = 0;
  scriptlist = 0;
  runninglist = 0;
  timerlist = 0;
  shortcutlist = 0;
  variablelist = 0;
  vartriglist = 0;
  statusvarlist = 0;
  gaugelist = 0;
  windowlist = 0;

  //load associated objects, if needed
  if (loadedFull)
  {
    //sess certainly isn't NULL
    QString fname = directory + "aliases";
    aliaslist = new cAliasList (fname, sess);
    fname = directory + "triggers";
    triggerlist = new cTriggerList (fname, sess);
    fname = directory + "actions";
    actionlist = new cActionList (fname, sess);
    actionlist->rebuildToolbar ();
    fname = directory + "aliasgroups";
    aliasgrouplist = new cGroupList (fname, sess, aliaslist);
    fname = directory + "triggergroups";
    triggergrouplist = new cGroupList (fname, sess, triggerlist);
    fname = directory + "scripts";
    scriptlist = new cScriptList (fname, sess);
    runninglist = new cRunningList (sess);
    fname = directory + "timers";
    timerlist = new cTimerList (fname, sess);
    timerlist->enableTimers ();
    fname = directory + "macrokeys";
    shortcutlist = new cShortcutList (fname, sess);
    fname = directory + "variables";
    variablelist = new cVariableList (fname, sess);
    fname = directory + "vartriggers";
    vartriglist = new cVarTriggerList (fname, sess);
    fname = directory + "statusvars";
    statusvarlist = new cStatusVarList (fname, sess);
    fname = directory + "gauges";
    gaugelist = new cGaugeList (fname, sess);
    fname = directory + "windowlist";
    windowlist = new cWindowList (fname, sess);

    //make variable triggers work
    QObject::connect (variablelist, SIGNAL (variableChanged (const QString &, const QString &)),
        vartriglist, SLOT (execTriggers (const QString &, const QString &)));
    //make status variables work
    QObject::connect (variablelist, SIGNAL (variableChanged (const QString &, const QString &)),
        statusvarlist, SLOT (variableChanged (const QString &, const QString &)));
    //make gauges work
    QObject::connect (variablelist, SIGNAL (variableChanged (const QString &, const QString &)),
        gaugelist, SLOT (variableChanged (const QString &, const QString &)));
  }
}

void cConnPrefs::save ()
{
  //prepare configfile
  KConfig *config = new KConfig (directory + "preferences");

  //save basic info
  config->setGroup ("General");
  config->writeEntry ("Server", _server);
  config->writeEntry ("Port", _port);
  config->writeEntry ("Login", _login);
  config->writeEntry ("Password", _password);

  //login sequence
  config->setGroup ("Login sequence");
  int lsc = _connstr.count();
  config->writeEntry ("Count", lsc);
  QStringList::iterator it;
  int num = 1;
  for (it = _connstr.begin(); it != _connstr.end(); ++it)
  {
    config->writeEntry ("Line " + QString::number(num), *it);
    num++;
  }
  
  //connection parameters
  config->setGroup ("Connection");
  config->writeEntry ("ANSI Colors", _ansicolors);
  config->writeEntry ("Limit triggers", _limittriggers);
  config->writeEntry ("Limit repeater", _limitrepeater);
  config->writeEntry ("Negotiate on startup", _startupneg);
  config->writeEntry ("Prompt label", _promptlabel);
  config->writeEntry ("LPMud style", _lpmudstyle);
  config->writeEntry ("Status prompt", _statusprompt);
  config->writeEntry ("Console prompt", _consoleprompt);
  config->writeEntry ("Auto logging", _autoadvtranscript);

  //commands
  config->setGroup ("Commands");
  config->writeEntry ("North", _cmd[North]);
  config->writeEntry ("NorthEast", _cmd[NorthEast]);
  config->writeEntry ("East", _cmd[East]);
  config->writeEntry ("SouthEast", _cmd[SouthEast]);
  config->writeEntry ("South", _cmd[South]);
  config->writeEntry ("SouthWest", _cmd[SouthWest]);
  config->writeEntry ("West", _cmd[West]);
  config->writeEntry ("NorthWest", _cmd[NorthWest]);
  config->writeEntry ("Up", _cmd[Up]);
  config->writeEntry ("Down", _cmd[Down]);
  config->writeEntry ("Quit", _cmdquit);
  
  //scripts
  config->setGroup ("Scripts");
  config->writeEntry ("Script directory", scriptdir);
  config->writeEntry ("Working directory", defworkdir);

  //directories (the above dirs should be here as well, but we don't want to
  //corrupt settings of existing users)
  config->setGroup ("Directories");
  config->writeEntry ("Transcript directory", transcriptdir);
  
#ifdef HAVE_SOUND
  //MSP
  config->setGroup ("Sound Protocol");
  config->writeEntry ("Use MSP", usemsp);
  config->writeEntry ("Always MSP", alwaysmsp);
  config->writeEntry ("Midline MSP", midlinemsp);
  config->writeEntry ("Path count", sounddirs.count());
  int i = 0;
  for (it = sounddirs.begin(); it != sounddirs.end(); ++it)
    config->writeEntry (QString("Path ") + QString::number(++i), *it);
#endif  //HAVE_SOUND

#ifdef HAVE_MXP
  //MXP
  config->setGroup ("MUD Extension Protocol");
  //how to use MXP? By default, we auto-detect it (i.e., we parse for MXP stuff, but we
  //remain in LOCKED mode by default, until changed by server)
  config->writeEntry ("Use MXP", usemxp);
  config->writeEntry ("Variable prefix", varprefix);
#endif  //HAVE_MXP
  
  delete config;  //this saves all data

  //save associated objects, if needed
  if (loadedFull)
  {
    //none of the lists needs to be saved
  }
}

void cConnPrefs::saveAll ()
{
  //basic stuff
  save ();
  
  aliaslist->save ();
  triggerlist->save ();
  actionlist->save ();
  aliasgrouplist->save ();
  triggergrouplist->save ();
  scriptlist->save ();
  timerlist->save ();
  shortcutlist->save ();
  variablelist->save ();
  vartriglist->save ();
  statusvarlist->save ();
  gaugelist->save ();
  windowlist->save();

  //plug-ins
  pluginmanager->event (sess, "save");
}

//some functions that get/set preferences; most of them are in header file

void cConnPrefs::setPort (int port)
{
  if ((port > 0) && (port <= 65535))
    _port = port;
}

void cConnPrefs::setAnsiColors (bool val)
{
  _ansicolors = val;
  if (sess != NULL)
    sess->ansiparser()->setUseAnsi (val);
}

void cConnPrefs::setCmd (int which, QString val)
{
  if ((which >= 0) && (which <= 9))
    _cmd[which] = val;
}

void cConnPrefs::setQuit (QString val)
{
  _cmdquit = val;
  if (sess != NULL)
    sess->telnet()->setQuitCommand (val);
}

void cConnPrefs::setPromptLabel (bool val) 
{ 
  _promptlabel = val; 
  if(sess != NULL)
    sess->console()->setPromptLabel (val);
}

void cConnPrefs::setNegotiateOnStartup (bool val)
{
  _startupneg = val;
  if (sess)
    sess->telnet()->setNegotiateOnStartup (val);
}

void cConnPrefs::setLPMudStyle(bool val)
{
  _lpmudstyle = val;
  if(sess != NULL)
    sess->telnet()->setLPMudStyle(val);
}

void cConnPrefs::setStatusPrompt(bool val)
{
  _statusprompt = val;
  if(sess != NULL && val == false)
    sess->status()->clearPartialLine();
}

#ifdef HAVE_MXP

void cConnPrefs::setUseMXP (int val)
{
  usemxp = val;
  if (sess)
    sess->telnet()->setMXPAllowed (val);
}

#endif

